package eu.jonahbauer.raytracing.render.renderer.pdf;

import eu.jonahbauer.raytracing.math.Vec3;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.random.RandomGenerator;

public record CosineProbabilityDensityFunction(@NotNull Vec3 normal) implements ProbabilityDensityFunction {

    public CosineProbabilityDensityFunction {
        Objects.requireNonNull(normal, "normal");
        normal = normal.unit();
    }

    @Override
    public double value(@NotNull Vec3 direction) {
        var cos = normal.times(direction.unit());
        return Math.max(0, cos / Math.PI);
    }

    @Override
    public @NotNull Vec3 generate(@NotNull RandomGenerator random) {
        var out = normal().plus(Vec3.random(random, true));
        return out.isNearZero() ? normal() : out;
    }
}
